package cnf

import (
	"bytes"
	"testing"
)

func TestIniGet(t *testing.T) {

	tests := []struct {
		raw string
		key string
	}{
		{"key = value", "key"},
		{"[section]\nkey = value", "section.key"},
		{"[ignoresec]\nignore = xxx\n[section]\nkey = value", "section.key"},
	}

	for _, test := range tests {

		parser := new(ini)
		reader := bytes.NewBufferString(test.raw)

		// try to extract value
		_, err := parser.ReadFrom(reader)
		if err != nil {
			t.Errorf("parse error: %s", err)
			continue
		}

		// extract value
		value, found := parser.Get(test.key)
		if !found {
			t.Errorf("expected a result, got none")
			continue
		}

		// check value
		if value != "value" {
			t.Errorf("expected 'value' got '%s'", value)
		}

	}

}

func TestIniSetPathExists(t *testing.T) {

	tests := []struct {
		raw   string
		key   string
		value string
	}{
		{"key = value", "key", "newvalue"},
		{"[section]\nkey = value", "section.key", "newvalue"},
		{"[ignoresec]\nignore = xxx\n[section]\nkey = value", "section.key", "newvalue"},
	}

	for _, test := range tests {

		parser := new(ini)
		reader := bytes.NewBufferString(test.raw)

		// try to extract value
		_, err := parser.ReadFrom(reader)
		if err != nil {
			t.Errorf("parse error: %s", err)
			continue
		}

		// update value
		if !parser.Set(test.key, test.value) {
			t.Errorf("cannot set '%s' to '%s'", test.key, test.value)
			continue
		}

		// check new value
		value, found := parser.Get(test.key)
		if !found {
			t.Errorf("expected a result, got none")
			continue
		}

		// check value
		if value != test.value {
			t.Errorf("expected '%s' got '%s'", test.value, value)
		}

	}

}

func TestIniSetCreatePath(t *testing.T) {

	tests := []struct {
		raw    string
		key    string
		ignore string // path to field that must be present after transformation
		value  string
	}{
		{"ignore = xxx", "key", "ignore", "newvalue"},
		{"ignore = xxx\n[section]\nkey = value", "section.key", "ignore", "newvalue"},
		{"[section]\nkey = value\nignore = xxx", "section.key", "section.ignore", "newvalue"},
		{"[ignoresec]\nignore = xxx\n[section]\nkey = value", "section.key", "ignoresec.ignore", "newvalue"},
	}

	for i, test := range tests {

		parser := new(ini)
		reader := bytes.NewBufferString(test.raw)

		// try to extract value
		_, err := parser.ReadFrom(reader)
		if err != nil {
			t.Errorf("[%d] parse error: %s", i, err)
			continue
		}

		// update value
		if !parser.Set(test.key, test.value) {
			t.Errorf("[%d] cannot set '%s' to '%s'", i, test.key, test.value)
			continue
		}

		// check new value
		value, found := parser.Get(test.key)
		if !found {
			t.Errorf("[%d] expected a result, got none", i)
			continue
		}

		// check value
		if value != test.value {
			t.Errorf("[%d] expected '%s' got '%s'", i, test.value, value)
			continue
		}

		// check that ignore field is still there
		value, found = parser.Get(test.ignore)
		if !found {
			t.Errorf("[%d] expected ignore field, got none", i)
			continue
		}

		// check value
		if value != "xxx" {
			t.Errorf("[%d] expected ignore value to be '%s' got '%s'", i, "xxx", value)
			continue
		}
	}

}
